#! /usr/local/bin/ruby
# -*- ruby -*-

#func Extraire les alias

#
# Retourne un tableau associatif (hash) qui
# associe  un alias la liste des destinataires.
#
def aliases
  # a est un tableau associatif vide au dpart
  a = {}
  # fd est un descripteur reprsentant le rsultat de
  # la commande ypcat -k aliases (ouvert en lecture)
  fd = open ("|ypcat -k aliases", "r")
  # Pour chaque ligne, que l'on appelle localement l
  fd.each_line {|l|
    # Enlever le caractre \n de fin de ligne
    l.chomp!
    # Reprer l'expression alias: nom1, nom2, nom3
    l =~ /^(\S+)\s(.*)$/
    # Renseigner le tableau associatif avec
    #  a['alias'] = ['nom1', 'nom2', 'nom3']
    a[$1] = $2.split /,\s*/
  }
  # On ferme le descripteur
  fd.close
  # On retourne le tableau
  return a
end

#
# Exemple:
#
#  a = aliases
#  a ["postmaster"]
#         => ["dax", "beyssac", "tardieu"]
#

#func Trouver les comptes correspondant  un alias

# Entre: l'alias  rechercher et le tableau associatif des alias
# Sortie: la liste des comptes qualifis dans lesquels se trouve ce compte

def cherche_aliases nom, al
  if al.key? nom then
    # Ce nom est prsent dans les alias, le rsoudre et aplatir
    # la liste (flatten). Mais d'abord, retirer le nom lui-mme
    # de la liste.
    dests = al[nom]
    dests.delete nom
    (dests.map {|a| cherche_aliases a, al}).flatten
  else
    [nom]
  end
end

#
# Exemple:
#
#  a = aliases
#  cherche_aliases "postmaster", a
#           => ["dax@inf", "beyssac@bofh", "tardieu@inf"]
#

#func Liste des alias associs  un compte

# Entre: le tableau associatif des alias
# Sortie: le tableau associant  chaque compte qualifi les alias
#         auxquels il appartient
def list_aliases aliases
  # Tableau associatif de rsultats, initialement vide
  r = {}
  # Pour chaque alias appel localement a
  aliases.keys.each {|a|
    # Pour chaque compte appel localement c associ  cet alias
    (cherche_aliases a, aliases).each {|c|
      # Si ce compte existe dans le tableau rsultat, rajouter
      # l'alias, sinon crer la liste initiale ( un lment pour l'instant)
      if r.key? c then
        r[c] << a
      else
        r[c] = [a]
      end
    }
  }
  return r
end

#
# Exemple:
#
#  a = aliases
#  r = list_aliases a
#  r['peyrade@dir']
#         => ["marc.peyrade", "enst.fr-administrative-contact", "peyrade"]
#

#func Extraire les comptes

#
# Retourne un tableau associatif contenant les comptes prsents dans
# la commande ypcat passwd et les prnom/nom associs (sous forme
# d'une chane unique).
#
def accounts
  # Tableau associatif vide initialement
  c = {}
  # fd est un descripteur reprsentant le rsultat de
  # la commande ypcat passwd (ouvert en lecture)
  fd = open ("|ypcat passwd", "r")
  # Pour chaque ligne, que l'on appelle localement l
  fd.each_line {|l|
    # Reprer nom:XXX:XXX:XXX:Prnom Nom etc.:...
    l =~ /^([^:]+):[^:]*:[^:]*:[^:]*:([^:]*):/
    # Enregister le prnom et le nom, en enlevant ce qu'il peut y
    # avoir aprs la virgule
    c[$1] = ($2.split ',')[0]
  }
  # On ferme le descripteur
  fd.close
  # On retourne le tableau associatif
  return c
end

#
# Exemple:
#
#  c = accounts
#  c['tardieu']
#         => "Samuel Tardieu"
#
